-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt
local M = {}

local drivetrain4wdSmoother = newExponentialSmoothing(10)
local wiperValSmoother = newExponentialSmoothing(5)
local wiperStalkSmoother = newExponentialSmoothing(5)
local keySmoother = newExponentialSmoothing(10)
local gear_A_Smoother = newExponentialSmoothing(10)

local queueWiperSwing = false
local wiperSwing = false
local wipersEnabled = 0
local wiperForward = false
local wiperTimer = 0
local wiperMaxTimer = 150
local playSnd = true

local function onInit()
    print("ANIMATED WIPERS MOD - Creating Sounds...")
    sounds.createSFXSource("art/sound/wiper_loop.ogg", 'AudioDefault3D', "WiperLoop", v.data.refNodes[0].ref)
    --sounds.createSFXSource("art/sound/wax_off.ogg", 'AudioDefault3D', "WiperOff", v.data.refNodes[0].ref)

    queueWiperSwing = false
    wiperSwing = false
    wipersEnabled = 0
    wiperForward = false
    wiperTimer = 0
    wiperMaxTimer = 150

    electrics.values['gear_A'] = 0
    electrics.values['keyTurn'] = 0
    electrics.values['drivetrain_4wd'] = 0
    electrics.values['wiperVal'] = 0
end

local function reset()
  onInit()
end

local initialized = false
local function updateGFX(dt)
    if(not initialized) then
        reset()
        initialized = true
    end
    if(electrics.values['rpm'] ~= nil and powertrain.getDevice("mainEngine") ~= nil) then
        if(electrics.values['rpm'] > 20 and powertrain.getDevice("mainEngine").starterEngagedCoef > 0) then
            electrics.values['keyTurn'] = 1
        elseif(electrics.values['rpm'] > 20) then
            electrics.values['keyTurn'] = 0.5
        else
            electrics.values['keyTurn'] = 0
        end
    end
    --electrics.values['keyTurn'] = (electrics.values['rpm'] > 20) and 1 or 0 

    -- applyForce(node1, node2, forceMagnitude)
    --obj:applyForce(thruster[2], thruster[1], thruster[3])

    electrics.values['wipers'] = 1-(wipersEnabled / 2)
    if(input.keys.Y and wiperKey == 0) then
        if(wipersEnabled == 0) then
            wipersEnabled = 1
            wiperMaxTimer = 150
            gui.message("Wipers: Slow", 5, "wiperMsg")
        elseif(wipersEnabled == 1) then
            wipersEnabled = 2
            wiperMaxTimer = 60
            gui.message("Wipers: Fast", 5, "wiperMsg")
        else
            wipersEnabled = 0

            gui.message("Wipers: Off", 5, "wiperMsg")
        end
        
        if(not queueWiperSwing and not wiperForward) then
            queueWiperSwing = true
            wiperForward = true
        end
        wiperTimer = 0
        wiperKey = 1
    end
    if(not input.keys.Y) then wiperKey = 0 end

    if(not wiperForward and wipersEnabled > 0) then
        wiperTimer = wiperTimer + 1
        if(wiperTimer > wiperMaxTimer) then
            queueWiperSwing = true
            wiperForward = true
            wiperTimer = 0
        end
    end
    if(not queueWiperSwing) then
        electrics.values['wiperVal'] = 0
        playSnd = true
    end

    if(queueWiperSwing) then
        if(playSnd) then
            print("Sound!!")
            sounds.playSoundOnceAtNode("WiperLoop", v.data.refNodes[0].ref, 1)
            playSnd = false
        end
        local lerpVal = 10
        local spd = math.abs(electrics.values['wiperVal'] - 1) / lerpVal
        if(spd > 0.02) then spd = 0.02 end

        if(wiperForward) then
            electrics.values['wiperVal'] = electrics.values['wiperVal'] + spd
            if(electrics.values['wiperVal'] >= 0.95) then wiperForward = false end
        else
            electrics.values['wiperVal'] = electrics.values['wiperVal'] - spd
            if(electrics.values['wiperVal'] < 0) then 
                queueWiperSwing = false
                wiperForward = false
            end
        end
    end
    if(electrics.values['mode4WD'] == 1 and electrics.values['modeRangeBox'] == 0) then
        electrics.values['drivetrain_4wd'] = 0.5
    elseif(electrics.values['mode4WD'] == 1 and electrics.values['modeRangeBox'] == 1) then
        electrics.values['drivetrain_4wd'] = 1
    else
        electrics.values['drivetrain_4wd'] = 0
    end

    electrics.values['gear_A'] = gear_A_Smoother:get(electrics.values['gear_A'])
    electrics.values['drivetrain_4wd'] = drivetrain4wdSmoother:get(electrics.values['drivetrain_4wd'])
    --electrics.values['wiperVal'] = wiperValSmoother:get(electrics.values['wiperVal'])
    electrics.values['wipers'] = wiperStalkSmoother:get(electrics.values['wipers'])
    electrics.values['keyTurn'] = keySmoother:get(electrics.values['keyTurn'])
end

-- public interface
M.onInit      = onInit
M.onReset     = onInit
M.updateGFX = updateGFX

return M